require 'json'
require 'retries'

require 'aws-sdk-ec2'


Puppet::Type.type(:aws_placement_group).provide(:arm) do
  mk_resource_methods

  def initialize(value = {})
    super(value)
    @property_flush = {}
    @is_create = false
    @is_delete = false
  end

  def namevar
    :group_name
  end

  # Properties

  def dry_run=(value)
    Puppet.info("dry_run setter called to change to #{value}")
    @property_flush[:dry_run] = value
  end

  def filters=(value)
    Puppet.info("filters setter called to change to #{value}")
    @property_flush[:filters] = value
  end

  def group_name=(value)
    Puppet.info("group_name setter called to change to #{value}")
    @property_flush[:group_name] = value
  end

  def group_names=(value)
    Puppet.info("group_names setter called to change to #{value}")
    @property_flush[:group_names] = value
  end

  def strategy=(value)
    Puppet.info("strategy setter called to change to #{value}")
    @property_flush[:strategy] = value
  end

  def name=(value)
    Puppet.info("name setter called to change to #{value}")
    @property_flush[:name] = value
  end

  def self.region
    ENV['AWS_REGION'] || 'us-west-2'
  end

  def self.name?(hash)
    !hash[:name].nil? && !hash[:name].empty?
  end


  def self.instances
    Puppet.debug("Calling instances for region #{region}")
    client = Aws::EC2::Client.new(region: region)

    all_instances = []
    client.describe_placement_groups.each do |response|
      response.placement_groups.each do |i|
        hash = instance_to_hash(i)
        all_instances << new(hash) if name?(hash)
      end
    end
    all_instances
  end

  def self.prefetch(resources)
    instances.each do |prov|
      tags = prov.respond_to?(:tags) ? prov.tags : nil
      tags = prov.respond_to?(:tag_set) ? prov.tag_set : tags
      next if tags.empty?
      name = tags.find { |x| x[:key] == 'Name' }[:value]
      if (resource = (resources.find { |k, _| k.casecmp(name).zero? } || [])[1])
        resource.provider = prov
      end
    end
  end

  def self.name_from_tag(instance)
    tags = instance.respond_to?(:tags) ? instance.tags : nil
    tags = instance.respond_to?(:tag_set) ? instance.tag_set : tags
    name = tags.find { |x| x.key == 'Name' } unless tags.nil?
    name.value unless name.nil?
  end

  def self.instance_to_hash(instance)
    dry_run = instance.respond_to?(:dry_run) ? (instance.dry_run.respond_to?(:to_hash) ? instance.dry_run.to_hash : instance.dry_run) : nil
    filters = instance.respond_to?(:filters) ? (instance.filters.respond_to?(:to_hash) ? instance.filters.to_hash : instance.filters) : nil
    group_name = instance.respond_to?(:group_name) ? (instance.group_name.respond_to?(:to_hash) ? instance.group_name.to_hash : instance.group_name) : nil
    group_names = instance.respond_to?(:group_names) ? (instance.group_names.respond_to?(:to_hash) ? instance.group_names.to_hash : instance.group_names) : nil
    strategy = instance.respond_to?(:strategy) ? (instance.strategy.respond_to?(:to_hash) ? instance.strategy.to_hash : instance.strategy) : nil

    hash = {}
    hash[:ensure] = :present
    hash[:object] = instance
    hash[:name] = name_from_tag(instance)
    hash[:tags] = instance.tags if instance.respond_to?(:tags) && !instance.tags.empty?
    hash[:tag_set] = instance.tag_set if instance.respond_to?(:tag_set) && !instance.tag_set.empty?

    hash[:dry_run] = dry_run unless dry_run.nil?
    hash[:filters] = filters unless filters.nil?
    hash[:group_name] = group_name unless group_name.nil?
    hash[:group_names] = group_names unless group_names.nil?
    hash[:strategy] = strategy unless strategy.nil?
    hash
  end

  def create
    @is_create = true
    Puppet.info("Entered create for resource #{resource[:name]} of type Instances")
    client = Aws::EC2::Client.new(region: self.class.region)
    response = client.create_placement_group(build_hash)
    res = response.respond_to?(:placement_group) ? response.placement_group : response
    with_retries(max_tries: 5) do
      client.create_tags(
        resources: [res.to_hash[namevar]],
        tags: [{ key: 'Name', value: resource.provider.name }],
      )
    end
    @property_hash[:ensure] = :present
  rescue StandardError => ex
    Puppet.alert("Exception during create. The state of the resource is unknown.  ex is #{ex} and backtrace is #{ex.backtrace}")
    raise
  end

  def flush
    Puppet.info("Entered flush for resource #{name} of type <no value> - creating ? #{@is_create}, deleting ? #{@is_delete}")
    if @is_create || @is_delete
      return # we've already done the create or delete
    end
    @is_update = true
    build_hash
    Puppet.info('Calling Update on flush')
    @property_hash[:ensure] = :present
  end

  def build_hash
    placement_group = {}
    if @is_create || @is_update
      placement_group[:dry_run] = resource[:dry_run] unless resource[:dry_run].nil?
      placement_group[:filters] = resource[:filters] unless resource[:filters].nil?
      placement_group[:group_name] = resource[:group_name] unless resource[:group_name].nil?
      placement_group[:group_names] = resource[:group_names] unless resource[:group_names].nil?
      placement_group[:group_name] = resource[:group_name] unless resource[:group_name].nil?
      placement_group[:strategy] = resource[:strategy] unless resource[:strategy].nil?
    end
    symbolize(placement_group)
  end

  def destroy
    Puppet.info("Entered delete for resource #{resource[:name]}")
    @is_delete = true
    Puppet.info('Calling operation delete_placement_group')
    client = Aws::EC2::Client.new(region: self.class.region)
    client.delete_placement_group(namevar => resource.provider.property_hash[namevar])
    @property_hash[:ensure] = :absent
  end


  # Shared funcs
  def exists?
    return_value = @property_hash[:ensure] && @property_hash[:ensure] != :absent
    Puppet.info("Checking if resource #{name} of type <no value> exists, returning #{return_value}")
    return_value
  end

  attr_reader :property_hash


  def symbolize(obj)
    return obj.reduce({}) do |memo, (k, v)|
      memo.tap { |m| m[k.to_sym] = symbolize(v) }
    end if obj.is_a? Hash

    return obj.reduce([]) do |memo, v|
      memo << symbolize(v); memo
    end if obj.is_a? Array
    obj
  end
end

# this is the end of the ruby class
